@@ -7,7 +7,8 @@
 #include "ImfRgbaFile.h"
 #include "ImfArray.h"
 #include "ImfChannelList.h"
-#include "ImfFrameBuffer.h"
+#include "ImfCompositeDeepScanLine.h"
+#include "ImfCompressor.h"
 #include "ImfDeepFrameBuffer.h"
 #include "ImfPartType.h"
 #include "ImfInputFile.h"
@@ -440,16 +441,23 @@ bool readDeepScanLine(T& in,bool reduceMemory, bool reduceTime)
 
             //
             // count how many samples are required to store this scanline
+            // in reduceMemory mode, pixels with large sample counts are not read,
+            // but the library needs to allocate memory for them internally
+            // - bufferSize is how much memory this function will allocate
+            // - fileBufferSize tracks how much decompressed data the library will require
             //
             size_t bufferSize = 0;
-            for (int j = 0; j < w ; j++)
+            size_t fileBufferSize = 0;
+            for (int j = 0; j < w; j++)
             {
                 for (int k = 0; k < channelCount; k++)
                 {
+                    fileBufferSize += localSampleCount[j];
                     //
                     // don't read samples which require a lot of memory in reduceMemory mode
                     //
-                    if (!reduceMemory || localSampleCount[j]*bytesPerSample <= gMaxBytesPerDeepPixel )
+                    if (!reduceMemory || localSampleCount[j] * bytesPerSample <=
+                                             gMaxBytesPerDeepPixel)
                     {
                         bufferSize += localSampleCount[j];
                     }
@@ -624,12 +632,19 @@ readDeepTile(T& in,bool reduceMemory , bool reduceTime)
 
 
                                 size_t bufferSize = 0;
+                                size_t fileBufferSize = 0;
 
                                 for (int ty = 0 ; ty < tileHeight ; ++ty )
                                 {
                                     for (int tx = 0 ; tx < tileWidth ; ++tx )
                                     {
-                                        if (!reduceMemory || localSampleCount[ty][tx]*bytesPerSample < gMaxBytesPerDeepScanline )
+                                        fileBufferSize += channelCount *
+                                                localSampleCount[ty][tx];
+
+                                        if (!reduceMemory ||
+                                            localSampleCount[ty][tx] *
+                                                    bytesPerSample <
+                                                gMaxBytesPerDeepScanline)
                                         {
                                             bufferSize += channelCount * localSampleCount[ty][tx];
                                         }
@@ -637,7 +652,10 @@ readDeepTile(T& in,bool reduceMemory , bool reduceTime)
                                 }
 
                                 // skip reading if no data to read, or limiting memory and tile is too large
-                                if ( bufferSize>0 && (!reduceMemory || bufferSize*bytesPerSample < gMaxBytesPerDeepPixel ))
+                                if (bufferSize > 0 &&
+                                    (!reduceMemory ||
+                                     (fileBufferSize + bufferSize) * bytesPerSample <
+                                         gMaxBytesPerDeepPixel))
                                 {
 
                                     pixelBuffer.resize( bufferSize );
@@ -1513,7 +1531,7 @@ memstream_read (
     {
         memdata *md = static_cast<memdata *>( userdata );
         uint64_t left = sz;
-        if ((offset + sz) > md->bytes)
+        if (offset > md->bytes ||  sz > md->bytes || offset+sz > md->bytes)
             left = (offset < md->bytes) ? md->bytes - offset : 0;
         if ( left > 0 )
             memcpy( buffer, md->data + offset, left );
@@ -1569,10 +1587,22 @@ bool
 checkOpenEXRFile (const char* fileName, bool reduceMemory, bool reduceTime, bool enableCoreCheck)
 {
     bool threw = false;
-    if ( enableCoreCheck )
+
+    uint64_t oldMaxSampleCount = CompositeDeepScanLine::getMaximumSampleCount();
+
+    if( reduceMemory || reduceTime)
+    {
+        CompositeDeepScanLine::setMaximumSampleCount(1<<20);
+    }
+
+    if (enableCoreCheck)
+    {
         threw = runCoreChecks (fileName, reduceMemory, reduceTime);
-    if (!threw)
-        threw = runChecks (fileName, reduceMemory, reduceTime);
+    }
+    if (!threw) threw = runChecks (fileName, reduceMemory, reduceTime);
+
+    CompositeDeepScanLine::setMaximumSampleCount(oldMaxSampleCount);
+
     return threw;
 }
 
@@ -1581,13 +1611,23 @@ bool
 checkOpenEXRFile (const char* data, size_t numBytes, bool reduceMemory, bool reduceTime, bool enableCoreCheck)
 {
     bool threw = false;
-    if ( enableCoreCheck )
+    uint64_t oldMaxSampleCount = CompositeDeepScanLine::getMaximumSampleCount();
+
+    if( reduceMemory || reduceTime)
+    {
+        CompositeDeepScanLine::setMaximumSampleCount(1<<20);
+    }
+
+    if (enableCoreCheck)
         threw = runCoreChecks (data, numBytes, reduceMemory, reduceTime);
     if (!threw)
     {
         PtrIStream stream (data,numBytes);
         threw = runChecks (stream, reduceMemory, reduceTime);
     }
+
+    CompositeDeepScanLine::setMaximumSampleCount(oldMaxSampleCount);
+
     return threw;
 }
 
